@@ -368,6 +368,10 @@ int skat_vkarte
 {
 	assert(pkr_self != pkr_who);
 	
+#ifndef NDEBUG
+	start_clock();
+#endif
+	
 	int type = -1;
 	VTMF_Card c;
 	char *tmp = new char[TMCG_MAX_CARD_CHARS];
@@ -447,6 +451,10 @@ int skat_vkarte
 	catch (int return_value)
 	{
 		delete [] tmp;
+#ifndef NDEBUG
+		stop_clock();
+		std::cerr << elapsed_time() << std::flush;
+#endif
 		return return_value;
 	}
 }
@@ -457,10 +465,22 @@ void skat_okarte
 		iosecuresocketstream *right, iosecuresocketstream *left
 	)
 {
+#ifndef NDEBUG
+	start_clock();
+#endif
+	
+	// use the non-interactiveness of the proof (only VTMF!)
+	std::stringstream proof;
+	tmcg->TMCG_ProveCardSecret(c, vtmf, proof, proof);
 	*right << c << std::endl << std::flush;
-	tmcg->TMCG_ProveCardSecret(c, vtmf, *right, *right);
+	*right << proof.str() << std::flush;
 	*left << c << std::endl << std::flush;
-	tmcg->TMCG_ProveCardSecret(c, vtmf, *left, *left);
+	*left << proof.str() << std::flush;
+	
+#ifndef NDEBUG
+	stop_clock();
+	std::cerr << elapsed_time() << std::flush;
+#endif
 }
 
 const char *skat_spiel2string
@@ -1013,6 +1033,10 @@ int skat_game
 	BarnettSmartVTMF_dlog *vtmf;
 	if (pkr_self == 2)
 	{
+#ifndef NDEBUG
+		start_clock();
+#endif
+		
 #ifdef COMMON_DDH_GROUP
 		std::stringstream ddh_group;
 		ddh_group << COMMON_DDH_GROUP << std::endl;
@@ -1046,9 +1070,20 @@ int skat_game
 				_("function KeyGenerationProtocol_UpdateKey() failed") << std::endl;
 			return 2;
 		}
+		
+		vtmf->KeyGenerationProtocol_Finalize();
+		
+#ifndef NDEBUG
+		stop_clock();
+		std::cerr << "KeyGenerationProtocol: " << elapsed_time() << std::endl;
+#endif
 	}
 	else
 	{
+#ifndef NDEBUG
+		start_clock();
+#endif
+		
 #ifdef COMMON_DDH_GROUP
 		std::stringstream ddh_group;
 		ddh_group << COMMON_DDH_GROUP << std::endl;
@@ -1103,6 +1138,13 @@ int skat_game
 			vtmf->KeyGenerationProtocol_PublishKey(*left);
 			vtmf->KeyGenerationProtocol_PublishKey(*right);
 		}
+		
+		vtmf->KeyGenerationProtocol_Finalize();
+		
+#ifndef NDEBUG
+		stop_clock();
+		std::cerr << "KeyGenerationProtocol: " << elapsed_time() << std::endl;
+#endif
 	}
 	
 	for (size_t r = 0; r < rounds; r++)
@@ -1136,6 +1178,9 @@ int skat_game
 				ost << nicks[pkr_self] << " MISCHEN" << std::endl;
 				*out_ctl << ost.str() << std::flush;
 			}
+#ifndef NDEBUG
+			start_clock();
+#endif
 			if (!skat_mischen(pkr_self, tmcg, vtmf, d2, ss,
 				d_mix[0], d_mix[1], d_mix[2], right, left))
 			{
@@ -1143,7 +1188,14 @@ int skat_game
 					_("bad stack format") << std::endl;
 				return 1;
 			}
+#ifndef NDEBUG
+			stop_clock();
+			std::cerr << elapsed_time() << std::flush;
+#endif
 			std::cout << "." << std::flush;
+#ifndef NDEBUG
+			start_clock();
+#endif
 			if (!skat_mischen_beweis(pkr_self, tmcg, vtmf, d2, ss,
 				d_mix[0], d_mix[1], d_mix[2], right, left))
 			{
@@ -1151,12 +1203,19 @@ int skat_game
 					_("wrong ZK proof") << std::endl;
 				return 2;
 			}
+#ifndef NDEBUG
+			stop_clock();
+			std::cerr << elapsed_time() << std::flush;
+#endif
 			std::cout << "." << _("finished!") << std::endl;
 			
 #ifdef ABHEBEN
 			// Abheben (cyclic shift of stack) von HH
 			std::cout << "><>< " << _("HH cuts the deck.") << " " <<
 				_("Please wait") << "." << std::flush;
+#ifndef NDEBUG
+			start_clock();
+#endif
 			if (((pkr_self + p) % 3) == 2)
 			{
 				size_t cyc = 0;
@@ -1175,7 +1234,6 @@ int skat_game
 				std::cout << "." << std::flush;
 				tmcg->TMCG_ProveStackEquality(d_mix[2], d_end, ab, true, vtmf,
 					*left, *left);
-				std::cout << "." << std::flush;
 				tmcg->TMCG_ProveStackEquality(d_mix[2], d_end, ab, true, vtmf,
 					*right, *right);
 			}
@@ -1213,8 +1271,12 @@ int skat_game
 					delete [] tmp, return 2;
 				}
 				delete [] tmp;
-				std::cout << "." << std::flush;
 			}
+#ifndef NDEBUG
+			stop_clock();
+			std::cerr << elapsed_time() << std::flush;
+#endif
+			std::cout << "." << std::flush;
 			std::cout << _("finished!") << std::endl;
 #else
 			d_end = d_mix[2];
@@ -1243,6 +1305,9 @@ int skat_game
 				ost << nicks[pkr_self] << " GEBEN" << std::endl;
 				*out_ctl << ost.str() << std::flush;
 			}
+#ifndef NDEBUG
+			start_clock();
+#endif
 			TMCG_Stack<VTMF_Card> s[3], sk;
 			if (!skat_geben(tmcg, d_end, s[0], s[1], s[2], sk))
 			{
@@ -1259,6 +1324,10 @@ int skat_game
 					_("wrong ZK proof") << std::endl;
 				return 4;
 			}
+#ifndef NDEBUG
+			stop_clock();
+			std::cerr << elapsed_time() << std::flush;
+#endif
 			std::cout << "." << _("finished!") << std::endl;
 			for (size_t i = 0; pctl && (i < os.size()); i++)
 			{
@@ -1411,7 +1480,8 @@ int skat_game
 							break;
 					}
 					
-					if ((msg.find("PASSE", 0) == 0) || (msg.find("passe", 0) == 0))
+					if ((msg.find("PASSE", 0) == 0) || (msg.find("passe", 0) == 0) ||
+						(msg.find("PASS", 0) == 0) || (msg.find("pass", 0) == 0))
 					{
 						if ((who_biding != 100) && (nick == nicks[who_biding]))
 						{
@@ -1460,7 +1530,8 @@ int skat_game
 							std::cout << ">< " << _("biding error") << ": " <<
 								_("pass action incorrect") << std::endl;
 					}
-					else if ((msg.find("REIZE", 0) == 0) || (msg.find("reize", 0) == 0))
+					else if ((msg.find("REIZE", 0) == 0) || (msg.find("reize", 0) == 0) ||
+						(msg.find("BID", 0) == 0) || (msg.find("bid", 0) == 0))
 					{
 						size_t rei = msg.find(" ", 6);
 						std::string srw = msg.substr(6, rei - 6);
@@ -1594,7 +1665,8 @@ int skat_game
 						}
 					}
 					else if ((msg.find("DRUECKE", 0) == 0) || 
-						(msg.find("druecke", 0) == 0))
+						(msg.find("druecke", 0) == 0) || (msg.find("PUSH", 0) == 0) || 
+						(msg.find("push", 0) == 0))
 					{
 						if ((reiz_status > 200) && (reiz_status < 300))
 						{
@@ -1611,17 +1683,17 @@ int skat_game
 										pkr.key[spiel_allein].name << 
 										"\" " << _("pushes the Skat") << std::endl;
 									if (pctl)
-										*out_ctl << nicks[spiel_allein] << " DRUECKE" <<
+										*out_ctl << nicks[spiel_allein] << " DRUECKE" << 
 											std::endl << std::flush;
-									if (((pkr_self == 0) && (spiel_allein == 1)) ||
-										((pkr_self == 1) && (spiel_allein == 2)) ||
+									if (((pkr_self == 0) && (spiel_allein == 1)) || 
+										((pkr_self == 1) && (spiel_allein == 2)) || 
 										((pkr_self == 2) && (spiel_allein == 0)))
 									{
 										left->getline(tmp1, TMCG_MAX_CARD_CHARS);
 										left->getline(tmp2, TMCG_MAX_CARD_CHARS);
 									}
-									else if (((pkr_self == 0) && (spiel_allein == 2)) ||
-										((pkr_self == 1) && (spiel_allein == 0)) ||
+									else if (((pkr_self == 0) && (spiel_allein == 2)) || 
+										((pkr_self == 1) && (spiel_allein == 0)) || 
 										((pkr_self == 2) && (spiel_allein == 1)))
 									{
 										right->getline(tmp1, TMCG_MAX_CARD_CHARS);
@@ -1661,7 +1733,8 @@ int skat_game
 						}
 					}
 					else if ((msg.find("SAGEAN", 0) == 0) || 
-						(msg.find("sagean", 0) == 0))
+						(msg.find("sagean", 0) == 0) || (msg.find("ANNOUNCE", 0) == 0) || 
+						(msg.find("announce", 0) == 0))
 					{
 						if ((!hand_spiel && (reiz_status > 300) && (reiz_status < 400))
 							|| (hand_spiel && (reiz_status > 200) && (reiz_status < 300)))
@@ -1731,7 +1804,8 @@ int skat_game
 						}
 					}	
 					else if ((msg.find("LEGE", 0) == 0) || 
-						(msg.find("lege", 0) == 0))
+						(msg.find("lege", 0) == 0) || (msg.find("PLAY", 0) == 0) || 
+						(msg.find("play", 0) == 0))
 					{
 						if (nick == nicks[spiel_who[spiel_dran]])
 						{
@@ -1802,7 +1876,8 @@ int skat_game
 					while (msg.find(" ", msg.length() - 1) == (msg.length() - 1))
 						msg = msg.substr(0, (msg.length() - 1));
 					
-					if ((msg.find("BLATT", 0) == 0) || (msg.find("blatt", 0) == 0))
+					if ((msg.find("BLATT", 0) == 0) || (msg.find("blatt", 0) == 0) || 
+						(msg.find("VIEW", 0) == 0) || (msg.find("view", 0) == 0))
 					{
 						skat_blatt((pkr_self + p) % 3, os);
 						if (os_ov.size() > 0)
@@ -1817,7 +1892,7 @@ int skat_game
 								std::cout << "MH";
 							if ((reiz_status - 412) == 2)
 								std::cout << "HH";
-							std::cout << ") " << _("announces") << " \"" << 
+							std::cout << ") " << _("announced") << " \"" << 
 								skat_spiel2string(spiel_status) << "\"" << std::endl;
 							std::cout << "><><>< " << _("played cards") << ": ";
 							for (size_t i = 0; i < os_sp.size(); i++)
@@ -1834,7 +1909,9 @@ int skat_game
 							}
 						}
 					}
-					else if ((msg.find("PASSE", 0) == 0) || (msg.find("passe", 0) == 0))
+					else if ((msg.find("PASSE", 0) == 0) ||
+						(msg.find("passe", 0) == 0) || (msg.find("PASS", 0) == 0) || 
+						(msg.find("pass", 0) == 0))
 					{
 						bool pass_ok = true;
 						switch ((pkr_self + p) % 3)
@@ -1906,7 +1983,9 @@ int skat_game
 							std::cout << ">< " << _("biding error") << ": " << 
 								_("passing is currently not allowed") << std::endl;
 					}
-					else if ((msg.find("REIZE", 0) == 0) || (msg.find("reize", 0) == 0))
+					else if ((msg.find("REIZE", 0) == 0) || 
+						(msg.find("reize", 0) == 0) || (msg.find("BID", 0) == 0) || 
+						(msg.find("bid", 0) == 0))
 					{
 					bool bid_ok = true;
 						switch ((pkr_self + p) % 3)
@@ -2050,7 +2129,8 @@ int skat_game
 								_("taking the Skat is currently not allowed") << std::endl;
 					}
 					else if ((msg.find("DRUECKE", 0) == 0) || 
-						(msg.find("druecke", 0) == 0))
+						(msg.find("druecke", 0) == 0) || (msg.find("PUSH", 0) == 0) || 
+						(msg.find("push", 0) == 0))
 					{
 						if ((reiz_status > 200) && (reiz_status < 300))
 						{
@@ -2059,16 +2139,18 @@ int skat_game
 								if (!hand_spiel)
 								{
 									std::string par = "";
-									size_t ei = par.npos;
-									if ((msg.find(" ", 0) == 7) && (msg.length() > 8))
+									size_t ei = par.npos, mi = msg.find(" ", 0);
+									
+									if ((mi != msg.npos) && (msg.length() > mi))
 									{
-										par = msg.substr(8, msg.length() - 8);
+										par = msg.substr(mi + 1, msg.length() - (mi + 1));
 										ei = par.find(" ", 0);
 									}
 									if ((ei != par.npos) && (par.length() > ei))
 									{
 										std::string cc1 = par.substr(0, ei);
-										std::string cc2 = par.substr(ei + 1, par.length() - ei - 1);
+										std::string cc2 = par.substr(ei + 1,
+											par.length() - (ei + 1));
 										
 										int tt1 = skat_wort2type(cc1), tt2 = skat_wort2type(cc2);
 										if ((tt1 != -1) && (tt2 != -1))
@@ -2125,27 +2207,29 @@ int skat_game
 								_("pushing the Skat is currently not allowed") << std::endl;
 					}
 					else if ((msg.find("SAGEAN", 0) == 0) || 
-						(msg.find("sagean", 0) == 0))
+						(msg.find("sagean", 0) == 0) || (msg.find("ANNOUNCE", 0) == 0) || 
+						(msg.find("announce", 0) == 0))
 					{
 						if ((!hand_spiel && (reiz_status > 300) && (reiz_status < 400))
 							|| (hand_spiel && (reiz_status > 200) && (reiz_status < 300)))
 						{
 							if (pkr_self == spiel_allein)
 							{
-								std::string par = ""; 
-								size_t ei = msg.find(" ", 0), zi = par.npos;
-								if ((ei == 6) && (msg.length() > 7))
+								std::string par = "";
+								size_t ei = par.npos, mi = msg.find(" ", 0);
+								
+								if ((mi != msg.npos) && (msg.length() > mi))
 								{
-									par = msg.substr(7, msg.length() - 7);
-									zi = par.find(" ", 0);
+									par = msg.substr(mi + 1, msg.length() - (mi + 1));
+									ei = par.find(" ", 0);
 								}
-								if ((ei != par.npos) && (par != ""))
+								if ((mi != msg.npos) && (par != ""))
 								{
 									std::string spiel = "", zusatz = "";
-									if (zi != par.npos)
+									if (ei != par.npos)
 									{
-										spiel = par.substr(0, zi);
-										zusatz = par.substr(zi + 1, par.length() - zi - 1);
+										spiel = par.substr(0, ei);
+										zusatz = par.substr(ei + 1, par.length() - (ei + 1));
 									}
 									else
 										spiel = par;
@@ -2176,6 +2260,8 @@ int skat_game
 														" :OUVERT " << skat_type2string(os[i].first) <<
 														" " << hex_game_digest << std::endl << std::flush;
 													skat_okarte(tmcg, vtmf, os[i].second, right, left);
+													// sleep to prevent "Excess Flood" error
+													sleep(3);
 												}
 											}
 											std::cout << "><><>< " << _("player") << " \"" << 
@@ -2198,7 +2284,7 @@ int skat_game
 											zusatz << "\"" << std::endl;
 								}
 								else
-									std::cout << ">< " << _("not enough parameter") << std::endl;
+									std::cout << ">< " << _("not enough parameters") << std::endl;
 							}
 							else
 								std::cout << ">< " << _("It's not your game.") << std::endl;
@@ -2207,13 +2293,17 @@ int skat_game
 							std::cout << ">< " << 
 								_("announcing the game is currently not allowed") << std::endl;
 					}
-					else if ((msg.find("LEGE", 0) == 0) || 
-						(msg.find("lege", 0) == 0))
+					else if ((msg.find("LEGE", 0) == 0) || (msg.find("lege", 0) == 0) || 
+						(msg.find("PLAY", 0) == 0) || (msg.find("play", 0) == 0))
 					{
 						if ((spiel_status > 0) && (pkr_self == spiel_who[spiel_dran]))
 						{
-							std::string par = 
-								(msg.length() > 5) ? msg.substr(5, msg.length() - 5) : "";
+							std::string par = "";
+							size_t mi = msg.find(" ", 0);
+							
+							if ((mi != msg.npos) && (msg.length() > mi))
+								par = msg.substr(mi + 1, msg.length() - (mi + 1));
+							
 							if (par.length() > 0)
 							{
 								int tt = skat_wort2type(par);
@@ -2226,7 +2316,7 @@ int skat_game
 											(!skat_rulectl(os_sp[0].first, tt, spiel_status, os)))
 										{
 											std::cout << ">< " << _("playing the card") << " \"" << 
-												par << "\" " << _("was not conform with the rules") << 
+												par << "\" " << _("is not conform with the rules") << 
 												std::endl;
 											continue;
 										}
@@ -2336,9 +2426,9 @@ int skat_game
 						pkt_gegner += pkt_wert[os_pkt[i][j].first];
 				pkt_allein = 120 - pkt_gegner;
 				std::cout << "><><>< " << _("card points") << " " <<
-					_("Alleinspieler") << " (" << pkr.key[spiel_allein].name << "): " << 
-					pkt_allein << ", " << _("card points") << " " << _("Gegenpartei") << 
-					": " << pkt_gegner << std::endl;
+					_("playing party") << " (" << pkr.key[spiel_allein].name << "): " << 
+					pkt_allein << ", " << _("card points") << " " << 
+					_("opponent party") << ": " << pkt_gegner << std::endl;
 				
 				if (pkt_allein > 60)
 					spiel_gewonnen = true;
@@ -2424,28 +2514,28 @@ int skat_game
 					// Schwarz gespielt (6)
 					if (os_pkt[spiel_allein].size() == 30)
 					{
-						std::cout << "><>< " << _("Gegenpartei") << " " << 
+						std::cout << "><>< " << _("opponent party") << " " << 
 							_("is") << " " << "Schwarz" << "!" << std::endl;
 						gstufen++;
 					}
 					// Schneider gespielt (7)
 					if (pkt_gegner < 31)
 					{
-						std::cout << "><>< " << _("Gegenpartei") << " " << 
+						std::cout << "><>< " << _("opponent party") << " " << 
 							_("is") << " " << "Schneider" << "!" << std::endl;
 						gstufen++;
 					}
 					// (selbst) Schwarz gespielt (6)
 					if (os_pkt[spiel_allein].size() == 0)
 					{
-						std::cout << "><>< " << _("Alleinspieler") << " " << _("is") << 
+						std::cout << "><>< " << _("playing party") << " " << _("is") << 
 							" " << "Schwarz" << ". " << _("Loosing the game!") << std::endl;
 						gstufen++, spiel_gewonnen = false;
 					}
 					// (selbst) Schneider gespielt (7)
 					if (pkt_allein < 31)
 					{
-						std::cout << "><>< " << _("Alleinspieler") << " " << _("is") << 
+						std::cout << "><>< " << _("playing party") << " " << _("is") << 
 							" " << "Schneider" << ". " << _("Loosing the game!") << std::endl;
 						gstufen++, spiel_gewonnen = false;
 					}
@@ -2454,7 +2544,7 @@ int skat_game
 				}
 				if (reiz_wert[reiz_counter] > (size_t)spiel_wert)
 				{
-					std::cout << "><>< " << _("Alleinspieler") << " " << 
+					std::cout << "><>< " << _("playing party") << " " << 
 						_("has bet to much") << ". " << _("Loosing the game!") << std::endl;
 					spiel_wert = 0;
 					while ((size_t)spiel_wert < reiz_wert[reiz_counter])
